@@ -72,6 +72,56 @@
 
 Q_LOGGING_CATEGORY(FOLDERMODEL, "plasma.containments.desktop.folder.foldermodel")
 
+
+class DragTrackerSingleton
+{
+public:
+    DragTracker self;
+};
+
+Q_GLOBAL_STATIC(DragTrackerSingleton, privateDragTrackerSelf)
+
+DragTracker::DragTracker(QObject *parent)
+    : QObject(parent)
+{
+}
+
+DragTracker::~DragTracker()
+{
+}
+
+bool DragTracker::isDragInProgress() const
+{
+    return m_dragInProgress;
+}
+
+void DragTracker::setDragInProgress(FolderModel *dragOwner, bool dragInProgress)
+{
+    if (dragInProgress == m_dragInProgress) {
+        return;
+    }
+
+    m_dragInProgress = dragInProgress;
+    if (dragInProgress) {
+        m_dragOwner = dragOwner;
+    } else {
+        m_dragOwner.clear();
+    }
+
+    Q_EMIT dragInProgressChanged(m_dragInProgress);
+}
+
+FolderModel *DragTracker::dragOwner()
+{
+    return m_dragOwner;
+}
+
+DragTracker *DragTracker::self()
+{
+    return &privateDragTrackerSelf()->self;
+}
+
+
 DirLister::DirLister(QObject *parent)
     : KDirLister(parent)
 {
@@ -94,7 +144,6 @@ void DirLister::handleError(KIO::Job *job)
 FolderModel::FolderModel(QObject *parent)
     : QSortFilterProxyModel(parent)
     , m_dirWatch(nullptr)
-    , m_dragInProgress(false)
     , m_urlChangedWhileDragging(false)
     , m_dropTargetPositionsCleanup(new QTimer(this))
     , m_previewGenerator(nullptr)
@@ -115,11 +164,13 @@ FolderModel::FolderModel(QObject *parent)
     , m_screenMapper(ScreenMapper::instance())
     , m_complete(false)
 {
+    connect(DragTracker::self(), &DragTracker::dragInProgressChanged, this, &FolderModel::draggingChanged);
+    connect(DragTracker::self(), &DragTracker::dragInProgressChanged, this, &FolderModel::dragInProgressAnywhereChanged);
     // needed to pass the job around with qml
     qmlRegisterType<KIO::DropJob>();
     DirLister *dirLister = new DirLister(this);
     dirLister->setDelayedMimeTypes(true);
-    dirLister->setAutoErrorHandlingEnabled(false, nullptr);
+    dirLister->setAutoErrorHandlingEnabled(false);
     connect(dirLister, &DirLister::error, this, &FolderModel::dirListFailed);
     connect(dirLister, &KCoreDirLister::itemsDeleted, this, &FolderModel::evictFromIsDirCache);
 
@@ -182,6 +233,7 @@ FolderModel::FolderModel(QObject *parent)
     });
 
     m_selectionModel = new QItemSelectionModel(this, this);
+    connect(m_selectionModel, &QItemSelectionModel::selectionChanged, this, &FolderModel::changeSelection);
     connect(m_selectionModel, &QItemSelectionModel::selectionChanged, this, &FolderModel::selectionChanged);
 
     setSourceModel(m_dirModel);
@@ -324,7 +376,7 @@ void FolderModel::setUrl(const QString &url)
         m_dirWatch->addFile(resolvedNewUrl.toLocalFile() + QLatin1String("/.directory"));
     }
 
-    if (m_dragInProgress) {
+    if (dragging()) {
         m_urlChangedWhileDragging = true;
     }
 
@@ -385,7 +437,12 @@ QString FolderModel::errorString() const
 
 bool FolderModel::dragging() const
 {
-    return m_dragInProgress;
+    return DragTracker::self()->isDragInProgress() && DragTracker::self()->dragOwner() == this;
+}
+
+bool FolderModel::isDragInProgressAnywhere() const
+{
+    return DragTracker::self()->isDragInProgress();
 }
 
 bool FolderModel::usedByContainment() const
@@ -608,7 +665,7 @@ void FolderModel::setFilterPattern(const QString &pattern)
     m_regExps.clear();
     m_regExps.reserve(patterns.count());
 
-    foreach (const QString &pattern, patterns) {
+    for (const QString &pattern : patterns) {
         QRegExp rx(pattern);
         rx.setPatternSyntax(QRegExp::Wildcard);
         rx.setCaseSensitivity(Qt::CaseInsensitive);
@@ -714,26 +771,28 @@ void FolderModel::run(int row)
 
 void FolderModel::runSelected()
 {
-    if (!m_selectionModel->hasSelection()) {
+    const QModelIndexList indexes = m_selectionModel->selectedIndexes();
+
+    if (indexes.isEmpty()) {
         return;
     }
 
-    if (m_selectionModel->selectedIndexes().count() == 1) {
-        run(m_selectionModel->selectedIndexes().constFirst().row());
+    if (indexes.count() == 1) {
+        run(indexes.first().row());
         return;
     }
 
     KFileItemActions fileItemActions(this);
     KFileItemList items;
 
-    foreach (const QModelIndex &index, m_selectionModel->selectedIndexes()) {
+    for (const QModelIndex &index : indexes) {
         // Skip over directories.
         if (!index.data(IsDirRole).toBool()) {
             items << itemForIndex(index);
         }
     }
 
-    fileItemActions.runPreferredApplications(items, QString());
+    fileItemActions.runPreferredApplications(items);
 }
 
 void FolderModel::rename(int row, const QString &name)
@@ -821,7 +880,7 @@ void FolderModel::updateSelection(const QVariantList &rows, bool toggle)
 
     int iRow = -1;
 
-    foreach (const QVariant &row, rows) {
+    for (const QVariant &row : rows) {
         iRow = row.toInt();
 
         if (iRow < 0) {
@@ -905,7 +964,7 @@ void FolderModel::addDragImage(QDrag *drag, int x, int y)
 
     QRegion region;
 
-    foreach (DragImage *image, m_dragImages) {
+    for (DragImage *image : std::as_const(m_dragImages)) {
         image->blank = isBlank(image->row);
         image->rect.translate(-m_dragHotSpotScrollOffset.x(), -m_dragHotSpotScrollOffset.y());
         if (!image->blank && !image->image.isNull()) {
@@ -924,7 +983,7 @@ void FolderModel::addDragImage(QDrag *drag, int x, int y)
 
     QPoint pos;
 
-    foreach (DragImage *image, m_dragImages) {
+    for (DragImage *image : std::as_const(m_dragImages)) {
         if (!image->blank && !image->image.isNull()) {
             pos = image->rect.translated(-offset.x(), -offset.y()).topLeft();
             image->cursorOffset.setX(pos.x() - (x - offset.x()));
@@ -943,12 +1002,11 @@ void FolderModel::addDragImage(QDrag *drag, int x, int y)
 
 void FolderModel::dragSelected(int x, int y)
 {
-    if (m_dragInProgress) {
+    if (dragging()) {
         return;
     }
 
-    m_dragInProgress = true;
-    Q_EMIT draggingChanged();
+    DragTracker::self()->setDragInProgress(this, true);
     m_urlChangedWhileDragging = false;
 
     // Avoid starting a drag synchronously in a mouse handler or interferes with
@@ -960,8 +1018,7 @@ void FolderModel::dragSelected(int x, int y)
 void FolderModel::dragSelectedInternal(int x, int y)
 {
     if (!m_viewAdapter || !m_selectionModel->hasSelection()) {
-        m_dragInProgress = false;
-        Q_EMIT draggingChanged();
+        DragTracker::self()->setDragInProgress(nullptr, false);
         return;
     }
 
@@ -977,11 +1034,11 @@ void FolderModel::dragSelectedInternal(int x, int y)
     std::sort(m_dragIndexes.begin(), m_dragIndexes.end());
 
     // TODO: Optimize to Q_EMIT contiguous groups.
-    Q_EMIT dataChanged(m_dragIndexes.first(), m_dragIndexes.last(), QVector<int>() << BlankRole);
+    Q_EMIT dataChanged(m_dragIndexes.constFirst(), m_dragIndexes.constLast(), {BlankRole});
 
     QModelIndexList sourceDragIndexes;
     sourceDragIndexes.reserve(m_dragIndexes.count());
-    foreach (const QModelIndex &index, m_dragIndexes) {
+    for (const QModelIndex &index : std::as_const(m_dragIndexes)) {
         sourceDragIndexes.append(mapToSource(index));
     }
 
@@ -998,8 +1055,7 @@ void FolderModel::dragSelectedInternal(int x, int y)
 
     item->ungrabMouse();
 
-    m_dragInProgress = false;
-    Q_EMIT draggingChanged();
+    DragTracker::self()->setDragInProgress(nullptr, false);
     m_urlChangedWhileDragging = false;
 
     if (m_dirModel->dirLister()->url() == currentUrl) {
@@ -1007,7 +1063,7 @@ void FolderModel::dragSelectedInternal(int x, int y)
         const QModelIndex last(m_dragIndexes.last());
         m_dragIndexes.clear();
         // TODO: Optimize to Q_EMIT contiguous groups.
-        Q_EMIT dataChanged(first, last, QVector<int>() << BlankRole);
+        Q_EMIT dataChanged(first, last, {BlankRole});
     }
 }
 
@@ -1105,7 +1161,7 @@ void FolderModel::drop(QQuickItem *target, QObject *dropEvent, int row, bool sho
     const int y = dropEvent->property("y").toInt();
     const QPoint dropPos = {x, y};
 
-    if (m_dragInProgress && row == -1 && !m_urlChangedWhileDragging) {
+    if (dragging() && row == -1 && !m_urlChangedWhileDragging) {
         if (m_locked || mimeData->urls().isEmpty()) {
             return;
         }
@@ -1256,23 +1312,23 @@ void FolderModel::dropCwd(QObject *dropEvent)
     }
 }
 
-void FolderModel::selectionChanged(const QItemSelection &selected, const QItemSelection &deselected)
+void FolderModel::changeSelection(const QItemSelection &selected, const QItemSelection &deselected)
 {
     QModelIndexList indices = selected.indexes();
     indices.append(deselected.indexes());
 
-    QVector<int> roles;
-    roles.append(SelectedRole);
+    const QVector<int> roles{SelectedRole};
 
-    foreach (const QModelIndex &index, indices) {
+    for (const QModelIndex &index : std::as_const(indices)) {
         Q_EMIT dataChanged(index, index, roles);
     }
 
     if (!m_selectionModel->hasSelection()) {
         clearDragImages();
     } else {
-        foreach (const QModelIndex &idx, deselected.indexes()) {
-            delete m_dragImages.take(idx.row());
+        const QModelIndexList deselectedIndices = deselected.indexes();
+        for (const QModelIndex &index : deselectedIndices) {
+            delete m_dragImages.take(index.row());
         }
     }
 
@@ -1388,7 +1444,7 @@ bool FolderModel::isDir(const QModelIndex &index, const KDirModel *dirModel) con
         KIO::StatJob *job = KIO::stat(url, KIO::HideProgressInfo);
         job->setProperty("org.kde.plasma.folder_url", item.url());
         job->setSide(KIO::StatJob::SourceSide);
-        job->setDetails(0);
+        job->setDetails(KIO::StatNoDetails);
         connect(job, &KJob::result, this, &FolderModel::statResult);
         m_isDirJobs.insert(item.url(), job);
     }
@@ -1414,7 +1470,7 @@ void FolderModel::statResult(KJob *job)
 
 void FolderModel::evictFromIsDirCache(const KFileItemList &items)
 {
-    foreach (const KFileItem &item, items) {
+    for (const KFileItem &item : items) {
         m_screenMapper->removeFromMap(item.url());
         m_isDirCache.remove(item.url());
     }
@@ -1739,6 +1795,8 @@ void FolderModel::updateActions()
 
 void FolderModel::openContextMenu(QQuickItem *visualParent, Qt::KeyboardModifiers modifiers)
 {
+    Q_UNUSED(modifiers)
+
     if (m_usedByContainment && !KAuthorized::authorize(QStringLiteral("action/kdesktop_rmb"))) {
         return;
     }
@@ -1785,7 +1843,7 @@ void FolderModel::openContextMenu(QQuickItem *visualParent, Qt::KeyboardModifier
         // Start adding the actions:
         // "Open" and "Open with" actions
         m_fileItemActions->setItemListProperties(itemProperties);
-        m_fileItemActions->addOpenWithActionsTo(menu);
+        m_fileItemActions->insertOpenWithActionsTo(nullptr, menu, QStringList());
         menu->addSeparator();
         menu->addAction(m_actionCollection.action(QStringLiteral("cut")));
         menu->addAction(m_actionCollection.action(QStringLiteral("copy")));
@@ -1849,14 +1907,19 @@ void FolderModel::openContextMenu(QQuickItem *visualParent, Qt::KeyboardModifier
     }
 
     menu->popup(m_menuPosition);
-    connect(menu, &QMenu::aboutToHide, [menu]() {
+    connect(menu, &QMenu::aboutToHide, [this, menu]() {
         menu->deleteLater();
+
+        // Remove the event filter for swapping delete and trash action from the QCoreApplication as it is no longer needed
+        if (RemoveAction *removeAction = qobject_cast<RemoveAction *>(m_actionCollection.action(QStringLiteral("remove"))))
+            QCoreApplication::instance()->removeEventFilter(removeAction);
     });
 }
 
 void FolderModel::openPropertiesDialog()
 {
     const QModelIndexList indexes = m_selectionModel->selectedIndexes();
+
     if (indexes.isEmpty()) {
         return;
     }
